//
//  LKFFileExecution.swift
//  lkfc
//
//  Created by Alexander Holzer on 11.06.18.
//  Copyright © 2018 Alexander Holzer. All rights reserved.
//

import Foundation

public protocol LKFBatchFileExecutionBackgroundProtocol {
    func finished (batchFileResult: LKFBatchFileExecutionResult)
}

public class LKFBatchFileExecution {
    
    var file: URL
    public var context: LKFContext?
    var caseStat : LKFCaseStat?
    
    var parameters : LKFBatchFileExecutionParameters
    
    public var packageSize : Int?
    
    public init (file: URL, parameters: LKFBatchFileExecutionParameters) {
        self.file = file
        self.parameters = parameters
    }
    
    public func execute (chunkSize: Int) -> LKFBatchFileExecutionResult {
        do {
            let context : LKFContext
            if let rootPath = parameters.rootPath {
            
                if parameters.statOnly {
                    context = LKFStatOnlyContext(year: 0, path: rootPath)
                }
                else {
                    context = LKFContext(model: 0, path: rootPath)
                }
                
            }
            else {
                context = LKFContext()
            }
            
            if parameters.keepContext {
                self.context = context
            }

            let statFile = LKFOutputStatFile(context:context)
            
            if !parameters.statOnly {
                context.add(file:LKFOutputDataFile(context:context))
                context.add(file:LKFOutputErrorFile(context:context))
            }
            
            context.add(file:statFile)
            
            if !context.executePackaged(packageSize: chunkSize, file: file, encoding: .ascii, enableComparing: parameters.compareMode != .disable) {
                return LKFBatchFileExecutionResultFailed (execution:self, failureText: "cannot execute file")
            }
        
            for f in context.packageOutputFiles! {
                let fileUrl = parameters.outPath.appendingPathComponent(f.defaultfileName, isDirectory: false)
                try context.write(file: fileUrl, lines: f.output ?? [])
            }
            caseStat = context.CaseStat()
            if let comparer = context.compareCache {
                
                let differences = comparer.compare()
                
                if differences != nil {
                    if parameters.compareMode == .print {
                        for s in differences! {
                            print(s + "\n")
                        }
                    }
                    return LKFBatchFileExecutionResultFailed (execution:self, failureText: "\(differences!.count) differences found")
                }
            }
            
            if let comparePath = parameters.comparePath {
                let statFileCompare = LKFTestFileCompare (file: statFile, comparePath: comparePath)
                if let statFileErrors = statFileCompare.compare() {
                    return LKFBatchFileExecutionResultFailed (execution:self, failureText: "\(statFileErrors.count) differences in stat file found")
                }
            }
        }
        catch {
            return LKFBatchFileExecutionResultFailed (execution:self, error:error)
        }
        return LKFBatchFileExecutionResult(execution: self)
    }
    
    public func execute() -> LKFBatchFileExecutionResult {
        
        if parameters.unloadStamm {
            LKFContext.unloadStamm()
        }
        
        do {
            let context : LKFContext
                
            if let rootPath = parameters.rootPath {
                if parameters.statOnly {
                    context = LKFStatOnlyContext(year: parameters.year, path: rootPath)
                }
                else {
                    context = LKFContext(model: parameters.model, path: rootPath)
                    if parameters.year > 0 {
                        context.setContextParameters(year: Int32(parameters.year))
                    }
                }
                if let contextFlags = parameters.contextFlags {
                    context.flags = contextFlags
                }
            }
            else {
                context = LKFContext()
            }

            if parameters.keepContext {
                self.context = context
            }

            if let packageSize = packageSize {
                if !context.executePackaged(packageSize: packageSize, file: file, encoding: .ascii, enableComparing: parameters.compareMode != .disable) {

                    if let lastError = context.lastError {
                        return LKFBatchFileExecutionResultFailed (execution:self, failureText: "cannot execute file: \(lastError.message)")
                    }
                    
                    return LKFBatchFileExecutionResultFailed (execution:self, failureText: "cannot execute file")
                }
            }
            else {
                try context.load(file: file, encoding:.ascii, enableComparing: parameters.compareMode != .disable)
                if !context.execute() {
                    
                    if let lastError = context.lastError {
                        return LKFBatchFileExecutionResultFailed (execution:self, failureText: "cannot execute file: \(lastError.message)")
                    }
                    
                    return LKFBatchFileExecutionResultFailed (execution:self, failureText: "cannot execute file")
                }
            }
            
            
            caseStat = context.CaseStat()
            
            if !parameters.statOnly {
                let outputDataFile = LKFOutputDataFile(context:context)
                let outputErrorFile = LKFOutputErrorFile(context: context)
                let outputWarningFile = LKFOutputWarningFile(context: context)
                let outputInfoFile = LKFOutputInfoFile(context: context)

                outputDataFile.createOutputForTest = parameters.createTestOutput
                let dataFile = parameters.outPath.appendingPathComponent(outputDataFile.defaultfileName, isDirectory: false)
                let errorFile = parameters.outPath.appendingPathComponent(outputErrorFile.defaultfileName, isDirectory: false)
                let warningFile = parameters.outPath.appendingPathComponent(outputWarningFile.defaultfileName, isDirectory: false)
                let infoFile = parameters.outPath.appendingPathComponent(outputInfoFile.defaultfileName, isDirectory: false)
                try context.write(file: dataFile, lines: outputDataFile.getOutput())
                if parameters.errorFiles {
                    try context.write(file: errorFile, lines: outputErrorFile.getOutput())
                    try context.write(file: warningFile, lines: outputWarningFile.getOutput())
                    try context.write(file: infoFile, lines: outputInfoFile.getOutput())
                }
            }

            let outputStatFile : LKFOutputStatFile?
            
            if parameters.includeStat || parameters.statOnly {
                outputStatFile = LKFOutputStatFile(context: context)
                let statFile = parameters.outPath.appendingPathComponent(outputStatFile!.defaultfileName, isDirectory: false)
                try context.write(file: statFile, lines: outputStatFile!.getOutput())
            } else {
                outputStatFile = nil
            }
            
            if parameters.statOnly == false, let comparer = context.compareCache {
                
                let differences = comparer.compare(removeBM: parameters.removeBM)
                
                if differences != nil {
                    if parameters.compareMode == .print {
                        for s in differences! {
                            print(s + "\n")
                        }
                    }
                    return LKFBatchFileExecutionResultFailed (execution:self, failureText: "\(differences!.count) differences found")
                }
                
                if outputStatFile != nil, let comparePath = parameters.comparePath {
                    let statFileCompare = LKFTestFileCompare (file: outputStatFile!, comparePath: comparePath)
                    if let statFileErrors = statFileCompare.compare() {
                        return LKFBatchFileExecutionResultFailed (execution:self, failureText: "\(statFileErrors.count) differences in stat file found")
                    }
                }
            }
        }
        catch {
            return LKFBatchFileExecutionResultFailed (execution:self, error:error)
        }
        return LKFBatchFileExecutionResult(execution: self)
    }
    
    public func threadExecute (delegate: LKFBatchFileExecutionBackgroundProtocol) {
        DispatchQueue.global(qos:.background).async {
            print ("starting execution: \(self.file)")
            let result = self.execute()
            print ("finished execution: \(self.file)")
            delegate.finished(batchFileResult: result)
        }
    }
    
    public func threadExecutePackaged (chunkSize: Int, delegate: LKFBatchFileExecutionBackgroundProtocol) {
        DispatchQueue.global(qos:.background).async {
            print ("starting execution: \(self.file)")
            let result = self.execute(chunkSize:chunkSize)
            print ("finished execution: \(self.file)")
            delegate.finished(batchFileResult: result)
        }
    }
}



